@@ -4,22 +4,17 @@
  *
  * Copyright (C) 2004 NVIDIA Corporation.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of Version 2 of the GNU General Public
- * License as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See Version 2
- * of the GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the:
- *
- *           Free Software Foundation, Inc.
- *           59 Temple Place - Suite 330
- *           Boston, MA 02111-1307, USA
- *
+ * along with this program.  If not, see <http://www.gnu.org/licenses>.
  */
 
 #ifndef __CTK_DISPLAYLAYOUT_H__
@@ -59,18 +54,7 @@ G_BEGIN_DECLS
 
 
 /* Maximums */
-#define MAX_DEVICES 8  /* Max number of GPUs */
-
-
-/* Rectangle/Dim data positions */
-#define LEFT         0
-#define TOP          1
-#define WIDTH        2
-#define HEIGHT       3
-#define X         LEFT
-#define Y          TOP
-#define W        WIDTH
-#define H       HEIGHT
+#define MAX_DEVICES 64  /* Max number of GPUs */
 
 
 /* XF86VIDMODE */
@@ -101,24 +85,11 @@ G_BEGIN_DECLS
   ((MODELINE_SOURCE_XCONFIG)|(MODELINE_SOURCE_NVCONTROL))
 
 
-/* NV-CONTROL metamode sources */
-#define METAMODE_SOURCE_XCONFIG   0x001
-#define METAMODE_SOURCE_IMPLICIT  0x002
-#define METAMODE_SOURCE_NVCONTROL 0x004
-
-#define METAMODE_SOURCE_USER  \
-  ((METAMODE_SOURCE_XCONFIG)|(METAMODE_SOURCE_NVCONTROL))
-
-
 
 
 /*** M A C R O S *************************************************************/
 
 
-#define NV_MIN(A, B) ((A)<(B)?(A):(B))
-#define NV_MAX(A, B) ((A)>(B)?(A):(B))
-
-
 /* Determines if the mode is the nvidia-auto-select mode. */
 #define IS_NVIDIA_DEFAULT_MODE(m)                        \
 (!strcmp(( m )->data.identifier, "nvidia-auto-select"))
@@ -128,11 +99,61 @@ G_BEGIN_DECLS
 #define GET_MODELINE_HSYNC(m)                                        \
 (((double)((m)->data.clock)) / (2.0f * (double)((m)->data.htotal)))
 
+/* Determines if the metamode was created/modified by the user */
+#define IS_METAMODE_SOURCE_USER(s)      \
+(((s) == METAMODE_SOURCE_XCONFIG) ||    \
+ ((s) == METAMODE_SOURCE_NVCONTROL) ||  \
+ ((s) == METAMODE_SOURCE_RANDR))
 
 
 
 /*** T Y P E   D E F I N I T I O N S *****************************************/
 
+typedef enum {
+    PASSIVE_STEREO_EYE_NONE = 0,
+    PASSIVE_STEREO_EYE_LEFT,
+    PASSIVE_STEREO_EYE_RIGHT,
+} PassiveStereoEye;
+
+typedef enum {
+    ROTATION_0 = 0,
+    ROTATION_90,
+    ROTATION_180,
+    ROTATION_270,
+} Rotation;
+
+typedef enum {
+    REFLECTION_NONE = 0,
+    REFLECTION_X,
+    REFLECTION_Y,
+    REFLECTION_XY,
+} Reflection;
+
+typedef enum {
+    METAMODE_SOURCE_XCONFIG = 0,
+    METAMODE_SOURCE_IMPLICIT,
+    METAMODE_SOURCE_NVCONTROL,
+    METAMODE_SOURCE_RANDR,
+} MetaModeSource;
+
+typedef enum {
+    MOSAIC_TYPE_UNSUPPORTED = 0,
+    MOSAIC_TYPE_SLI_MOSAIC,
+    MOSAIC_TYPE_BASE_MOSAIC,
+    MOSAIC_TYPE_BASE_MOSAIC_LIMITED,
+} MosaicType;
+
+typedef enum {
+    PIXELSHIFT_NONE = 0,
+    PIXELSHIFT_4K_TOP_LEFT,
+    PIXELSHIFT_4K_BOTTOM_RIGHT,
+    PIXELSHIFT_8K,
+} PixelShift;
+
+typedef struct nvSizeRec {
+    int width;
+    int height;
+} nvSize;
 
 typedef struct nvModeLineRec {
     struct nvModeLineRec *next;
@@ -149,11 +170,33 @@ typedef struct nvModeLineRec {
 
 
 
-/* Mode (A particular configuration for a display within an X screen) */
+typedef struct nvSelectedModeRec {
+    struct nvSelectedModeRec *next;
+
+    gchar *text;            /* Text shown in dropdown menu */
+
+    nvModeLinePtr modeline; /* Modeline this mode references */
+
+    Bool isSpecial;         /* Whether this mode is "Off" or "Auto" */
+    Bool isScaled;          /* Whether custom viewports are set */
+
+    nvSize viewPortIn;
+    GdkRectangle viewPortOut;
+} nvSelectedMode, *nvSelectedModePtr;
+
+
+
+/* Mode (A particular configuration for a display within an X screen)
+ *
+ * NOTE: When metamodes are duplicated, the modes are memcpy'ed over, so
+ *       if new variables are added to the nvModeRec that shouldn't (just)
+ *       be copied, be sure to update this in
+ *       ctk_display_layout_add_screen_metamode().
+ */
 typedef struct nvModeRec {
     struct nvModeRec *next;
 
-    /* Defines a single mode for a dispay device as part of an X screen's
+    /* Defines a single mode for a display device as part of an X screen's
      * metamode.
      *
      * "WxH_Hz +x+y @WxH"
@@ -166,31 +209,53 @@ typedef struct nvModeRec {
     struct nvModeLineRec *modeline;     /* Modeline this mode references */
     int dummy;                          /* Dummy mode, don't print out */
 
-    int dim[4];                         /* Viewport (absolute) */
-    int pan[4];                         /* Panning Domain (absolute) */
+    nvSize viewPortIn;                  /* Viewport In */
+    GdkRectangle pan;                   /* Panning Domain (absolute) */
+
+    GdkRectangle viewPortOut;           /* ViewPort Out (WH) */
 
     int position_type;                  /* Relative, Absolute, etc. */
     struct nvDisplayRec *relative_to;   /* Display Relative/RightOf etc */
 
+    PassiveStereoEye passive_stereo_eye; /* Stereo mode 4 per-dpy setting */
+
+
+    Rotation rotation;
+    Reflection reflection;
+    PixelShift pixelshift;
+
 } nvMode, *nvModePtr;
 
 
 
 /* Display Device (CRT, DFP, TV, Projector ...) */
 typedef struct nvDisplayRec {
-    struct nvDisplayRec *next;
+    struct nvDisplayRec *next_on_gpu;
+    struct nvDisplayRec *next_in_screen;
     XConfigMonitorPtr    conf_monitor;
 
+    CtrlTarget         *ctrl_target;
+
     struct nvGpuRec    *gpu;            /* GPU the display belongs to */
     struct nvScreenRec *screen;         /* X screen the display is tied to */
 
-    unsigned int        device_mask;    /* Bit mask to identify the display */
-    char               *name;           /* Display name (from NV-CONTROL) */
+    char               *logName;        /* Display name (from NV-CONTROL) */
+    char               *typeBaseName;   /* e.g. "CRT", "DFP", "TV" */
+    char               *typeIdName;     /* e.g. "DFP-1", "TV-0" */
+    char               *dpGuidName;     /* e.g. "DP-GUID-11111111-1111-1111-1111-111111111111" */
+    char               *edidHashName;   /* e.g. "DPY-EDID-11111111-1111-1111-1111-111111111111" */
+    char               *targetIdName;   /* e.g. "DPY-3" */
+    char               *randrName;      /* e.g. "VGA-1", "DVI-I-2" */
+
     Bool                is_sdi;         /* Is an SDI display */
 
     nvModeLinePtr       modelines;      /* Modelines validated by X */
     int                 num_modelines;
 
+    nvSelectedModePtr   selected_modes; /* List of modes to show in the dropdown menu */
+    int                 num_selected_modes;
+    nvSelectedModePtr   cur_selected_mode; /* Current mode selected in the dropdown menu */
+
     nvModePtr           modes;          /* List of modes this display uses */
     int                 num_modes;
     nvModePtr           cur_mode;       /* Current mode display uses */
@@ -204,16 +269,20 @@ typedef struct nvMetaModeRec {
     struct nvMetaModeRec *next;
 
     int id;     /* Magic id */
-    int source; /* Source of the metamode */
+    int x_idx;  /* Used to re-order metamodes on apply */
+    MetaModeSource source; /* Source of the metamode */
     Bool switchable; /* Can the metamode be accessed through Ctrl Alt +- */
 
     // Used for drawing & moving metamode boxes
-    int dim[4]; /* Bounding box of all modes */
+    GdkRectangle dim; /* Bounding box of all modes */
 
     // Used for applying and generating metamodes (effective dimensions)
-    int edim[4]; /* Bounding box of all non-NULL modes */
+    GdkRectangle edim; /* Bounding box of all non-NULL modes */
 
-    char *string; /* Temp string used for modifying the metamode list */
+    /* Used to apply the metamode to the running X server */
+    char *cpl_str; /* metamode string from CPL */
+    char *x_str;   /* parsed CPL string from X */
+    char *x_str_entry; /* Points to string in metamode strings buffer */
 
 } nvMetaMode, *nvMetaModePtr;
 
@@ -221,7 +290,8 @@ typedef struct nvMetaModeRec {
 
 /* X Screen */
 typedef struct nvScreenRec {
-    struct nvScreenRec *next;
+    struct nvScreenRec *next_in_layout;
+
     XConfigScreenPtr conf_screen;
     XConfigDevicePtr conf_device;
 
@@ -233,15 +303,29 @@ typedef struct nvScreenRec {
      *
      */
 
-    NvCtrlAttributeHandle *handle;  /* NV-CONTROL handle to X screen */
+    CtrlTarget *ctrl_target;
     CtkEvent *ctk_event;
     int scrnum;
 
-    struct nvGpuRec *gpu;  /* GPU driving this X screen */
+    struct nvLayoutRec *layout; /* Layout this X screen belongs to */
+    struct nvGpuRec **gpus; /* List of GPUs driving this screen */
+    int num_gpus;
+
+    int display_owner_gpu_id; /* Display owner GPU, or -1 */
+    struct nvGpuRec *display_owner_gpu; /* GPU to use for Device section */
+
+    int max_width;  /* Max based on all GPUs */
+    int max_height;
+    int max_displays;
+    Bool allow_depth_30;
 
     int depth;      /* Depth of the screen */
+    int stereo;     /* Stereo mode enabled on this screen */
+    int overlay;    /* Overlay enabled on this screen */
+    int hw_overlay;
+    int ubb;
 
-    unsigned int displays_mask; /* Display devices on this X screen */
+    nvDisplayPtr displays; /* List of displays using this screen */
     int num_displays; /* # of displays using this screen */
 
     nvMetaModePtr metamodes;     /* List of metamodes */
@@ -250,7 +334,7 @@ typedef struct nvScreenRec {
     int cur_metamode_idx;        /* Current metamode to display */
     nvDisplayPtr primaryDisplay;
     // Used for generating metamode strings.
-    int dim[4]; /* Bounding box of all metamodes (Absolute coords) */
+    GdkRectangle dim; /* Bounding box of all metamodes (Absolute coords) */
 
     int position_type;                /* Relative, Absolute, etc. */
     struct nvScreenRec *relative_to;  /* Screen Relative/RightOf etc */
@@ -258,8 +342,10 @@ typedef struct nvScreenRec {
     int y_offset;
 
     Bool sli;
-    Bool dynamic_twinview;  /* This screen supports dynamic twinview */
+    char *sli_mode;
+    char *multigpu_mode;
     Bool no_scanout;        /* This screen has no display devices */
+    Bool stereo_supported;  /* Can stereo be configured on this screen */
 
 } nvScreen, *nvScreenPtr;
 
@@ -276,31 +362,35 @@ typedef struct GvoModeDataRec {
 
 /* GPU (Device) */
 typedef struct nvGpuRec {
-    struct nvGpuRec *next;
+    struct nvGpuRec *next_in_layout; /* List of all GPUs */
 
-    NvCtrlAttributeHandle *handle;  /* NV-CONTROL handle to GPU */
+    CtrlTarget *ctrl_target;
     CtkEvent *ctk_event;
-    
+
     struct nvLayoutRec *layout; /* Layout this GPU belongs to */
 
     int max_width;
     int max_height;
     int max_displays;
     Bool allow_depth_30;
+    Bool multigpu_master_possible;
+
+    MosaicType mosaic_type;
+    Bool mosaic_enabled;
 
     char *name;  /* Name of the GPU */
-    
-    unsigned int connected_displays;  /* Bitmask of connected displays */
+    char *uuid;  /* e.g. "GPU-11111111-1111-1111-1111-111111111111" */
 
     gchar *pci_bus_id;
 
     GvoModeData *gvo_mode_data; /* Information about GVO modes available */
     unsigned int num_gvo_modes;
 
-    nvScreenPtr screens;  /* List of screens this GPU drives */
-    int num_screens;
+    unsigned int *flags_memory; /* Pointer to memory alloced for flags */
+    unsigned int *flags; /* Array of flags queried from the X server */
+    int num_flags;
 
-    nvDisplayPtr displays;  /* List of displays attached to screen */
+    nvDisplayPtr displays; /* Linked list of displays connected to GPU */
     int num_displays;
 
 } nvGpu, *nvGpuPtr;
@@ -312,13 +402,18 @@ typedef struct nvLayoutRec {
     XConfigLayoutPtr conf_layout;
     char *filename;
 
-    NvCtrlAttributeHandle *handle;
+    CtrlSystemList systems; /* Holds 1 system */
+    CtrlSystem *system;
 
-    nvGpuPtr gpus;  /* List of GPUs in the layout */
+
+    nvGpuPtr gpus;  /* Linked list of GPUs (next_in_layout) */
     int num_gpus;
 
-    // Used for drawing the layout.
-    int dim[4]; /* Bounding box of All X screens (Absolute coords) */
+    nvScreenPtr screens; /* Linked list of X screens (next_in_layout) */
+    int num_screens;
+
+    /* Used for drawing the layout */
+    GdkRectangle dim; /* Bounding box of All X screens (Absolute coords) */
 
     int xinerama_enabled;
 
@@ -339,14 +434,13 @@ typedef struct ModifyInfoRec {
     /* What to move */
     nvDisplayPtr display;
     nvScreenPtr screen;
-    int orig_screen_dim[4]; // Used when moding display = moding screen.
-    nvGpuPtr gpu;
+    GdkRectangle orig_screen_dim; // Used when moding display = moding screen.
 
     int orig_position_type; // Original values of what is being
-    int orig_dim[4];        // modified.
+    GdkRectangle orig_dim;  // modified.
 
     int *target_position_type; // Pointers to values of thing that
-    int *target_dim;           // is being modified.
+    GdkRectangle *target_dim;  // is being modified.
 
 
     /* Snapping */
@@ -354,12 +448,12 @@ typedef struct ModifyInfoRec {
     int best_snap_v;
     int best_snap_h;
 
-    int modify_dirty;   // Sync the modify_dim before moving/panning.
-    int modify_panning; // Modifying panning (instead of position)
-    int modify_dim[4];  // Dimensions to snap from
+    int modify_dirty;         // Sync the modify_dim before moving/panning.
+    int modify_panning;       // Modifying panning (instead of position)
+    GdkRectangle modify_dim;  // Dimensions to snap from
 
-    int src_dim[4]; // Pre-snap (To allow snapping of pan box on move)
-    int dst_dim[4]; // Post-snap position
+    GdkRectangle src_dim; // Pre-snap (To allow snapping of pan box on move)
+    GdkRectangle dst_dim; // Post-snap position
 
 } ModifyInfo;
 
@@ -385,25 +479,27 @@ typedef struct _CtkDisplayLayout
 {
     GtkVBox parent;
 
-    NvCtrlAttributeHandle *handle;
     CtkConfig             *ctk_config;
 
     GtkWidget   *drawing_area;   /* Drawing area */
     GtkWidget   *tooltip_area;   /* Tooltip area */
+#ifndef CTK_GTK3
     GtkTooltips *tooltip_group;  /* Tooltip group */
+#endif
 
     /* Layout configuration */
     nvLayoutPtr layout;
 
     /* Double buffering of layout image */
+#ifdef CTK_GTK3
+    cairo_t *c_context;
+#else
     GdkPixmap *pixmap;
-    int        need_swap;
+#endif
 
     /* Image information */
-    int        width;           /* Real widget dimensions */
-    int        height;
-    int        img_dim[4];      /* Dimensions used to draw in */
-    float      scale;
+    GdkRectangle img_dim;
+    float scale;
 
     /* Colors */
     GdkColor  *color_palettes;  /* Colors to use to display screens */
@@ -414,7 +510,7 @@ typedef struct _CtkDisplayLayout
     /* Pango layout for strings in layout image */
 
     PangoLayout *pango_layout;
-    
+
     /* List of visible elements in the layout */
     ZNode *Zorder; /* Z ordering of visible elements in layout */
     int    Zcount; /* Count of visible elements in the z order */
@@ -429,8 +525,6 @@ typedef struct _CtkDisplayLayout
                                 /* - multiple modes */
 
     /* State */
-    void      *first_selected_display; /* First thing clicked on (used to cycle) */
-    void      *first_selected_screen;
     int        clicked_outside; /* User clicked outside displays, don't move */
     ModifyInfo modify_info;     /* Used to move/pan screens/displays */
 
@@ -459,8 +553,7 @@ typedef struct _CtkDisplayLayoutClass
 
 GType       ctk_display_layout_get_type  (void) G_GNUC_CONST;
 
-GtkWidget*  ctk_display_layout_new       (NvCtrlAttributeHandle *,
-                                          CtkConfig *,
+GtkWidget*  ctk_display_layout_new       (CtkConfig *,
                                           nvLayoutPtr, /* Layout to display */
                                           int,         /* Width of image */
                                           int          /* Height of image */
@@ -468,21 +561,28 @@ GtkWidget*  ctk_display_layout_new       (NvCtrlAttributeHandle *,
 
 void ctk_display_layout_update (CtkDisplayLayout *);
 
-void ctk_display_layout_redraw (CtkDisplayLayout *);
-
-
 void ctk_display_layout_set_layout (CtkDisplayLayout *, nvLayoutPtr);
 
 void ctk_display_layout_update_zorder(CtkDisplayLayout *ctk_object);
 
 nvDisplayPtr ctk_display_layout_get_selected_display (CtkDisplayLayout *);
 nvScreenPtr ctk_display_layout_get_selected_screen (CtkDisplayLayout *);
-nvGpuPtr ctk_display_layout_get_selected_gpu (CtkDisplayLayout *);
 
 
-void ctk_display_layout_set_mode_modeline (CtkDisplayLayout *,
-                                           nvModePtr mode,
-                                           nvModeLinePtr modeline);
+void ctk_display_layout_set_mode_modeline(CtkDisplayLayout *,
+                                          nvModePtr mode,
+                                          nvModeLinePtr modeline,
+                                          const nvSize *viewPortIn,
+                                          const GdkRectangle *viewPortOut);
+
+void ctk_display_layout_set_mode_viewport_in(CtkDisplayLayout *ctk_object,
+                                             nvModePtr mode,
+                                             int w, int h,
+                                             Bool update_panning_size);
+
+void ctk_display_layout_set_mode_viewport_out(CtkDisplayLayout *ctk_object,
+                                              nvModePtr mode,
+                                              int x, int y, int w, int h);
 
 void ctk_display_layout_set_display_position (CtkDisplayLayout *ctk_object,
                                               nvDisplayPtr display,
@@ -492,8 +592,19 @@ void ctk_display_layout_set_display_position (CtkDisplayLayout *ctk_object,
 void ctk_display_layout_set_display_panning (CtkDisplayLayout *ctk_object,
                                              nvDisplayPtr display,
                                              int width, int height);
+
+void ctk_display_layout_set_display_rotation (CtkDisplayLayout *ctk_object,
+                                              nvDisplayPtr display,
+                                              Rotation rotation);
+
+void ctk_display_layout_set_display_reflection (CtkDisplayLayout *ctk_object,
+                                                nvDisplayPtr display,
+                                                Reflection reflection);
+
 void ctk_display_layout_select_display (CtkDisplayLayout *ctk_object,
                                         nvDisplayPtr display);
+void ctk_display_layout_select_screen (CtkDisplayLayout *ctk_object,
+                                       nvScreenPtr screen);
 void ctk_display_layout_update_display_count (CtkDisplayLayout *,
                                               nvDisplayPtr);
 
