@@ -332,6 +332,8 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_SIGNAL(NV_CTRL_GSYNC_ALLOWED);
     MAKE_SIGNAL(NV_CTRL_GPU_NVCLOCK_OFFSET);
     MAKE_SIGNAL(NV_CTRL_GPU_MEM_TRANSFER_RATE_OFFSET);
+    MAKE_SIGNAL(NV_CTRL_GPU_NVCLOCK_OFFSET_ALL_PERFORMANCE_LEVELS);
+    MAKE_SIGNAL(NV_CTRL_GPU_MEM_TRANSFER_RATE_OFFSET_ALL_PERFORMANCE_LEVELS);
     MAKE_SIGNAL(NV_CTRL_VIDEO_DECODER_UTILIZATION);
     MAKE_SIGNAL(NV_CTRL_GPU_OVER_VOLTAGE_OFFSET);
     MAKE_SIGNAL(NV_CTRL_GPU_CURRENT_CORE_VOLTAGE);
@@ -351,7 +353,7 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
      * knows about.
      */
 
-#if NV_CTRL_LAST_ATTRIBUTE != NV_CTRL_DISPLAYPORT_SINK_IS_AUDIO_CAPABLE
+#if NV_CTRL_LAST_ATTRIBUTE != NV_CTRL_GPU_MEM_TRANSFER_RATE_OFFSET_ALL_PERFORMANCE_LEVELS
 #warning "There are attributes that do not emit signals!"
 #endif
 
